import java.awt.*;
import java.awt.event.*;

class DrawWindow implements Drawable, MouseListener, MouseMotionListener {
    private double      xMin;
    private double      yMin;
    private double      xMax;
    private double      yMax;
    private double      scale;
    private Canvas      canvas;
    private DrawWindowListener  listener;

    public DrawWindow(Canvas canvas,
                      double xMin, double yMin, double xMax, double yMax) {
        this.canvas = canvas;
        this.xMin = xMin;
        this.yMin = yMin;
        this.xMax = xMax;
        this.yMax = yMax;
    }

    // [hWnfoCXWnɕϊ
    private int wcToDcX(double wX) {
        return (int)((wX - this.xMin)* scale);
    }
    private int wcToDcY(double wY) {
        return (int)((wY - this.yMin) * scale);
    }
    private int wcToDcLength(double wLength) {
        return (int)(wLength * this.scale);
    }

    // foCXWn烏[hWnɕϊ
    private double dcToWcX(int dX) {
        return dX / this.scale + this.xMin;
    }
    private double dcToWcY(int dY) {
        return dY / this.scale + this.yMin;
    }

    // DrawWindow\z_ł͂܂Canvas\\ȏԂ
    // ȂߑΉGraphics擾łȂB
    // ŁAۂGraphicsKvɂȂ鎞_܂Graphics̎擾
    // xB
    private Graphics graphics = null;
    private Graphics getGraphics() {
        if (this.graphics == null) {
            this.graphics = this.canvas.getGraphics();
        }
        return this.graphics;
    }
    public void drawLine(double x1, double y1, double x2, double y2) {
        this.getGraphics().drawLine(wcToDcX(x1), wcToDcY(y1),
                                    wcToDcX(x2), wcToDcY(y2));
    }
    public void drawCircle(double centerX, double centerY, double radius) {
        int x = wcToDcX(centerX - radius);
        int y = wcToDcY(centerY - radius);
        int width = wcToDcLength(radius) * 2;
        int height = width;

        this.getGraphics().drawOval(x, y, width, height);
    }
    public void drawRectangle(double x1, double y1, double x2, double y2) {
        int     x;
        int     y;
        int     width;
        int     height;

        if (x1 < x2) {
            x = wcToDcX(x1);
            width = wcToDcLength(x2 - x1);
        } else {
            x = wcToDcX(x2);
            width = wcToDcLength(x1 - x2);
        }
        if (y1 < y2) {
            y = wcToDcY(y1);
            height = wcToDcLength(y2 - y1);
        } else {
            y = wcToDcY(y2);
            height = wcToDcLength(y1 - y2);
        }

        this.getGraphics().drawRect(x, y, width, height);
    }
    public void setXORMode(boolean isXORMode) {
        if (isXORMode) {
            this.getGraphics().setColor(Color.blue);
            this.getGraphics().setXORMode(Color.white);
        } else {
            this.getGraphics().setColor(Color.black);
            this.getGraphics().setPaintMode();
        }
    }
    private void adjustBoundary() {
        Dimension canvasSize = canvas.getSize();
        double xScale = canvasSize.width / (this.xMax - this.xMin);
        double yScale = canvasSize.height / (this.yMax - this.yMin);

        if (xScale < yScale) {
            this.scale = xScale;
        } else {
            this.scale = yScale;
        }
    }
    public void setDrawWindowListener(DrawWindowListener listener) {
        this.listener = listener;
    }
    public void mouseDown(int x, int y) {}
    public void mouseUp(int x, int y) {}
    public void mouseDrag(int x, int y) {}
    public void mousePressed(MouseEvent e) {
        if (this.listener == null) {
            return;
        }
        int     button;
        if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
            button = DrawWindowListener.BUTTON1;
        } else if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
            button = DrawWindowListener.BUTTON2;
        } else {
            return;
        }
        this.listener.mouseButtonPressed(this,
                                         dcToWcX(e.getX()), dcToWcX(e.getY()),
                                         button);
    }
    public void mouseReleased(MouseEvent e) {
        if (this.listener == null) {
            return;
        }
        
        int     button;
        if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
            button = DrawWindowListener.BUTTON1;
        } else if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
            button = DrawWindowListener.BUTTON2;
        } else {
            return;
        }
        listener.mouseButtonReleased(this,
                                     dcToWcX(e.getX()), dcToWcX(e.getY()),
                                     button);
    }
    public void mouseDragged(MouseEvent e) {
        if (this.listener == null) {
            return;
        }
        this.listener.mouseMoved(this,
                                 dcToWcX(e.getX()), dcToWcX(e.getY()));
    }
    public void mouseMoved(MouseEvent e) {
        if (this.listener == null) {
            return;
        }
        this.listener.mouseMoved(this,
                                 dcToWcX(e.getX()), dcToWcX(e.getY()));
    }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

    public void resize(int width, int height) {
        this.adjustBoundary();
    }
}
