class PolylineCommand extends Command {
    private static final int INITIAL_MODE = 0;
    private static final int DRAWING_START_MODE = 1;
    private static final int DRAWING_MODE = 2;
    private int currentMode;
    private Point2D[] points;
    private double prevX;
    private double prevY;

    PolylineCommand(ShapeCollection shapeCollection) {
        super(shapeCollection);
        this.currentMode = INITIAL_MODE;
    }
    private static Point2D[] expandArray(Point2D[] src) {
        Point2D[] newArray = new Point2D[src.length + 1];
        for (int i = 0; i < src.length; i++) {
            newArray[i] = src[i];
        }
        return newArray;
    }
    public void mouseButtonPressed(DrawWindow d, double x, double y,
                                   int button) {
        if (this.currentMode == INITIAL_MODE
            && button == DrawWindowListener.BUTTON1) {
            this.points = new Point2D[1];
            this.points[0] = new Point2D(x, y);
            this.currentMode = DRAWING_START_MODE;

        } else if (this.currentMode == DRAWING_MODE) {
            drawRubberBand(d, this.prevX, this.prevY);
            d.drawLine(this.points[this.points.length-1].getX(),
                       this.points[this.points.length-1].getY(),
                       x, y);

            this.points = expandArray(this.points);
            this.points[this.points.length-1] = new Point2D(x, y);
            
            if (button == DrawWindowListener.BUTTON2) {
                shapeCollection.addShape(new Polyline(this.points));
                this.currentMode = INITIAL_MODE;
            } else {
                this.currentMode = DRAWING_START_MODE;
            }
        }
    }
    public void mouseButtonReleased(DrawWindow d, double x, double y,
                                    int button) {
    }
    private void drawRubberBand(DrawWindow d, double x, double y) {
        d.setXORMode(true);
        d.drawLine(this.points[this.points.length-1].getX(),
                   this.points[this.points.length-1].getY(),
                   x, y);
        d.setXORMode(false);
    }
    public void mouseMoved(DrawWindow d, double x, double y) {
        if (this.currentMode == INITIAL_MODE)
            return;

        if (this.currentMode == DRAWING_MODE) {
            drawRubberBand(d, this.prevX, this.prevY);
        } else {
            this.currentMode = DRAWING_MODE;
        }
        drawRubberBand(d, x, y);
        this.prevX = x;
        this.prevY = y;
        
    }
    public void redraw(DrawWindow d) {
        if (this.currentMode != INITIAL_MODE) {
            for (int i = 0; i < this.points.length - 1; i++) {
                d.drawLine(this.points[i].getX(), this.points[i].getY(),
                           this.points[i+1].getX(), this.points[i+1].getY());
            }
        }
        if (this.currentMode == DRAWING_MODE) {
            drawRubberBand(d, this.prevX, this.prevY);
        }
    }
    public void finishCommand() {
        this.currentMode = INITIAL_MODE;
    }
}
