import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

public class XDrawApplet extends Applet {
    private XDrawCanvas         canvas;
    private ShapeCollection     shapeCollection;

    public void init() {
        Panel           buttonPanel;
        Button          clearButton;
        Button          polylineButton;
        Button          circleButton;
        Button          rectangleButton;

        this.setLayout(new BorderLayout());

        buttonPanel = new Panel();
        this.add(BorderLayout.NORTH, buttonPanel);

        clearButton = new Button("Clear");
        buttonPanel.add(clearButton);
        clearButton.addActionListener(new ClearButtonListener());

        polylineButton = new Button("Polyine");
        buttonPanel.add(polylineButton);
        polylineButton.addActionListener(new PolylineButtonListener());

        circleButton = new Button("Circle");
        buttonPanel.add(circleButton);
        circleButton.addActionListener(new CircleButtonListener());

        rectangleButton = new Button("Rectangle");
        buttonPanel.add(rectangleButton);
        rectangleButton.addActionListener(new RectangleButtonListener());

        this.shapeCollection = new ShapeCollection();

        this.canvas = new XDrawCanvas(this.shapeCollection);
        this.add(BorderLayout.CENTER, canvas);
    }
    private class ClearButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            XDrawApplet.this.shapeCollection.clear();
            canvas.repaint();
        }
    }
    private class PolylineButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            canvas.changeCurrentCommand(new PolylineCommand(shapeCollection));
        }
    }
    private class CircleButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            canvas.changeCurrentCommand(new CircleCommand(shapeCollection));
        }
    }
    private class RectangleButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            canvas.changeCurrentCommand(new RectangleCommand(shapeCollection));
        }
    }
    public static void main(String[] args) {
        XDrawFrame app
            = new XDrawFrame("XDraw");
        app.setSize(800, 500);
        app.setVisible(true);
    }
    private static class XDrawFrame extends Frame {
        private XDrawApplet applet;

        public XDrawFrame(String name) {
            super(name);
            addWindowListener(new XDrawWindowAdapter());
            this.applet = new XDrawApplet();
            this.applet.init();
            this.applet.start();
            add("Center", applet);
        }
        private class XDrawWindowAdapter extends WindowAdapter {
            public void windowClosing(WindowEvent e) {
                applet.stop();
                applet.destroy();
                System.exit(0);
            }
        }
    }
}
