class CircleCommand extends Command {
    private static final int INITIAL_MODE = 0;
    private static final int DRAGGING_START_MODE = 1;
    private static final int DRAGGING_MODE = 2;
    private double centerX;
    private double centerY;
    private double prevRadius;
    private int currentMode;

    CircleCommand(ShapeCollection shapeCollection) {
        super(shapeCollection);
        this.currentMode = INITIAL_MODE;
    }
    public void mouseButtonPressed(DrawWindow d, double x, double y,
                                   int button) {
        this.centerX = x;
        this.centerY = y;
        this.currentMode = DRAGGING_START_MODE;
    }
    public void mouseButtonReleased(DrawWindow d, double x, double y,
                                    int button) {
        if (this.currentMode != DRAGGING_MODE) {
            this.currentMode = INITIAL_MODE;
            return;
        }
        d.setXORMode(true);
        d.drawCircle(this.centerX, this.centerY, this.prevRadius);
        d.setXORMode(false);

        double radius = calcDistance(this.centerX, this.centerY, x, y);
        d.drawCircle(this.centerX, this.centerY, radius);
        
        shapeCollection.addShape(new Circle(this.centerX, this.centerY,
                                            radius));
        this.currentMode = INITIAL_MODE;
    }
    private double calcDistance(double x1, double y1, double x2, double y2) {
        return Math.sqrt((x1 - x2) * (x1 - x2)
                         + (y1 - y2) * (y1 - y2));
    }
    public void mouseMoved(DrawWindow d, double x, double y) {
        if (this.currentMode == INITIAL_MODE)
            return;

        d.setXORMode(true);
        if (this.currentMode == DRAGGING_MODE) {
            d.drawCircle(this.centerX, this.centerY, this.prevRadius);
        } else {
            this.currentMode = DRAGGING_MODE;
        }
        double radius = calcDistance(this.centerX, this.centerY, x, y);
        d.drawCircle(this.centerX, this.centerY, radius);
        d.setXORMode(false);

        this.prevRadius = radius;
    }
}
