import java.awt.*;
import java.awt.event.*;

class XDrawCanvas extends Canvas {
    private DrawWindow          drawWindow;
    private Command             currentCommand;
    private ShapeCollection     shapeCollection;

    public XDrawCanvas(ShapeCollection shapeCollection) {
        this.drawWindow = new DrawWindow(this, 0, 0, 100, 100);

        this.addMouseListener(this.drawWindow);
        this.addMouseMotionListener(this.drawWindow);
        this.addComponentListener(new CanvasComponentListener());

        this.shapeCollection = shapeCollection;
        this.setBackground(Color.white);
    }
    public void paint(Graphics g) {
        ShapeIterator iterator = shapeCollection.iterator();
        while (iterator.hasNext()) {
            Shape shape = iterator.next();
            shape.draw(this.drawWindow);
        }
        if (this.currentCommand != null) {
            this.currentCommand.redraw(this.drawWindow);
        }
    }
    public void changeCurrentCommand(Command newCommand) {
        if (this.currentCommand != null) {
            this.currentCommand.finishCommand();
            repaint();
        }
        this.drawWindow.setDrawWindowListener(newCommand);
        this.currentCommand = newCommand;
    }
    class CanvasComponentListener implements ComponentListener {
        public void componentResized(ComponentEvent e) {
            Dimension d = getSize();
            drawWindow.resize(d.width, d.height);
            repaint();
        }
        public void componentMoved(ComponentEvent e) {}
        public void componentShown(ComponentEvent e) {}
        public void componentHidden(ComponentEvent e) {}
    }
}
